﻿package be.streamhead.boxes
{
	import caurina.transitions.Tweener;
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import sandy.core.Scene3D;
	import sandy.core.scenegraph.Camera3D;
	import sandy.core.scenegraph.Group;
	import sandy.materials.Appearance;
	import sandy.materials.attributes.LightAttributes;
	import sandy.materials.attributes.LineAttributes;
	import sandy.materials.attributes.MaterialAttributes;
	import sandy.materials.BitmapMaterial;
	import sandy.materials.ColorMaterial;
	import sandy.materials.Material;
	import sandy.primitive.Box;
	import sandy.primitive.Cone;
	import sandy.primitive.Hedra;
	import sandy.primitive.Plane3D;
	
	/**
	 * This file is part of a short introductory tutorial on using Sandy 3D in FlashDevelop.
	 * All information can be found at:
	 * 		http://www.streamhead.com/sandy-3d-tutorialsandy-3d-tutorial/
	 * @author Peter Backx
	 */
	public class Main extends Sprite 
	{
		// image credit: http://qvectors.com/nature/water-drops/
		[Embed(source = '../../../../lib/aqua.png')]
		private var aquaClass:Class;
		private var aquaBitmap:Bitmap = new aquaClass();
		
		[Embed(source = '../../../../lib/streamhead.png')]
		private var logoClass:Class;
		private var logoBitmap:Bitmap = new logoClass();
		
		[Embed(source = '../../../../lib/glasses.png')]
		private var glassesClass:Class;
		private var glassesBitmap:Bitmap = new glassesClass();
		
		[Embed(source = '../../../../lib/radialbeams.png')]
		private var beamsClass:Class;
		private var beamsBitmap:Bitmap = new beamsClass();
		
		private var scene:Scene3D;
		private var camera:Camera3D;
		private var myBox:Box;
		private var plane:Plane3D;
		private var g:Group;
		
		private var zoomedIn:Boolean = false;
		
		public function Main():void 
		{
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private function init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			// entry point
			camera = new Camera3D( 300, 300 );
			camera.x = 0;
			camera.y = 50;
			camera.z = -200;
			camera.lookAt(0,0,0);
      
			var root:Group = createScene();
      
			scene = new Scene3D( "scene", this, camera, root );
      
			addEventListener( Event.ENTER_FRAME, enterFrameHandler );
		}

		private function createScene():Group 
		{
			g = new Group();
      
			myBox = new Box( "theBox", 60, 60, 60 );
      
			myBox.container.buttonMode = true;
      
			var material:Material = new BitmapMaterial(logoBitmap.bitmapData);
			material.lightingEnable = true;
			var app:Appearance = new Appearance( material );
			var i:int;
			for (i = 0; i < 4; i++ )
				myBox.aPolygons[i].appearance = app;

			material = new BitmapMaterial(aquaBitmap.bitmapData);
			material.lightingEnable = true;
			app = new Appearance( material );
			for (i = 4; i < 8; i++ )
				myBox.aPolygons[i].appearance = app;

			material = new BitmapMaterial(glassesBitmap.bitmapData);
			material.lightingEnable = true;
			app = new Appearance( material );
			for (i = 8; i < 12; i++ )
				myBox.aPolygons[i].appearance = app;

      
			myBox.container.addEventListener(MouseEvent.CLICK, clickHandler);
			myBox.container.addEventListener(MouseEvent.MOUSE_OUT, outHandler);
			myBox.container.addEventListener(MouseEvent.MOUSE_OVER, overHandler);
			
			g.addChild(myBox);
			
			plane = new Plane3D( "thePlane", 200, 200);
			plane.z = 50;
			material = new BitmapMaterial(beamsBitmap.bitmapData);
			material.lightingEnable = true;
			plane.appearance = new Appearance(material);
			
			return g;
		}

		private function enterFrameHandler( event : Event ) : void 
		{
			if(!zoomedIn) {
				myBox.tilt += 1;
				myBox.pan += 4;
			}
			scene.render();
		}

		private function clickHandler(event:MouseEvent):void {
			if (!zoomedIn) {
				g.removeChildByName("thePlane");
				Tweener.addTween(myBox, {z:-100, time:2, transition:"easeOutBounce"});
				zoomedIn = true;
			} else {
				Tweener.addTween(myBox, {z:0, time:2, transition:"easeInOutCubic"});
				zoomedIn = false;
			}
		}

		private function overHandler(event:MouseEvent):void {
			if(!zoomedIn)
				g.addChild(plane);
		}

		private function outHandler(event:MouseEvent):void {
			g.removeChildByName("thePlane");
		}

	}
	
}